/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package fr.amap.lidar.amapvox.voxelisation.output;

import fr.amap.lidar.amapvox.voxelisation.Voxelization;
import fr.amap.lidar.amapvox.voxelisation.Voxelization.GroundEnergy;
import fr.amap.lidar.amapvox.voxelisation.configuration.params.GroundEnergyParams;
import java.awt.Color;
import java.awt.color.ColorSpace;
import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import javax.imageio.ImageIO;

/**
 *
 * @author pverley
 */
public class GroundEnergyOutput extends AbstractOutput {

    public GroundEnergyOutput(Voxelization voxelisation) {
        super(voxelisation);
    }

    @Override
    public File[] write() throws IOException {

        GroundEnergy[][] groundEnergy = voxelisation.getGroundEnergy();

        if (groundEnergy.length > 0 && groundEnergy[0].length > 0) {
            
            File outputFile = voxParam.getGroundEnergyParams().getGroundEnergyFile();
            String fileName = outputFile.getName();

            LOGGER.info("writing file " + outputFile.getAbsolutePath());
            int ivox = 0;
            int nvox = voxParam.infos.getSplit().x * voxParam.infos.getSplit().y;

            if (voxParam.getGroundEnergyParams().getGroundEnergyFileFormat() == GroundEnergyParams.FILE_FORMAT_PNG) {

                BufferedImage image = new BufferedImage(voxParam.infos.getSplit().x, voxParam.infos.getSplit().y, BufferedImage.TYPE_INT_ARGB);

                for (int i = 0; i < voxParam.infos.getSplit().x; i++) {
                    for (int j = 0; j < voxParam.infos.getSplit().y; j++) {
                        progress("writing file " + fileName, ivox++, nvox);
                        
                        float transmittance = groundEnergy[i][j].groundEnergyActual / groundEnergy[i][j].groundEnergyPotential;

                        Color c;

                        if (transmittance <= 1.0 && transmittance >= 0.0) {
                            c = new Color(ColorSpace.getInstance(ColorSpace.CS_GRAY), new float[]{transmittance}, 1.0f);

                        } else if (transmittance > 1.0) {
                            c = new Color(1.0f, 0.0f, 0.0f, 1.0f);
                        } else if (transmittance < 0.0) {
                            c = new Color(0.0f, 1.0f, 0.0f, 1.0f);
                        } else {
                            c = new Color(0.0f, 0.0f, 1.0f, 1.0f);
                        }

                        image.setRGB(i, voxParam.infos.getSplit().y - 1 - j, c.getRGB());

                    }
                }

                ImageIO.write(image, "png", new File(voxParam.getGroundEnergyParams().getGroundEnergyFile().getAbsolutePath()));

            } else {
                try (BufferedWriter writer = new BufferedWriter(new FileWriter(outputFile))) {
                    writer.write("i j groundEnergyActual groundEnergyPotential transmittance\n");

                    for (int i = 0; i < voxParam.infos.getSplit().x; i++) {
                        for (int j = 0; j < voxParam.infos.getSplit().y; j++) {
                            progress("writing file " + fileName, ivox++, nvox);

                            float transmittance = groundEnergy[i][j].groundEnergyActual / groundEnergy[i][j].groundEnergyPotential;
                            writer.write(i + " " + j + " " + groundEnergy[i][j].groundEnergyActual + " " + groundEnergy[i][j].groundEnergyPotential + " " + transmittance + "\n");
                        }
                    }
                }
            }

            return new File[] {outputFile};

        } else {
            LOGGER.warn("Ground energy is set up but no values are available");
        }

        return null;
    }

}
